@@ -2,32 +2,34 @@
 # under the Affero General Public License v3, see <https://www.gnu.org/licenses/>.
 
 from abc import ABC, abstractmethod
+from copy import deepcopy
 from typing import (
     Any,
     Callable,
     Dict,
     List,
     Optional,
-    Sequence,
     Tuple,
-    TypeVar,
-    Union,
-    cast,
 )
 
 import numpy as np
 import torch
+from math import ceil
 from pyro.infer.mcmc import HMC, NUTS
 from pyro.infer.mcmc.api import MCMC
-from torch import Tensor, log
+from torch import Tensor
 from torch import multiprocessing as mp
 from torch import nn
 
 from sbi import utils as utils
-from sbi.mcmc import Slice, SliceSampler
+from sbi.mcmc import Slice, SliceSampler, SliceSamplerVectorized, prior_init, sir
 from sbi.types import Array, Shape
 from sbi.user_input.user_input_checks import process_x
-from sbi.utils.torchutils import atleast_2d_float32_tensor, ensure_theta_batched
+from sbi.utils.torchutils import (
+    ScalarFloat,
+    atleast_2d_float32_tensor,
+    ensure_theta_batched,
+)
 
 
 class NeuralPosterior(ABC):
@@ -47,7 +49,6 @@ class NeuralPosterior(ABC):
         x_shape: torch.Size,
         mcmc_method: str = "slice_np",
         mcmc_parameters: Optional[Dict[str, Any]] = None,
-        get_potential_function: Optional[Callable] = None,
     ):
         """
         Args:
@@ -65,10 +66,8 @@ class NeuralPosterior(ABC):
                 samples to discard, `num_chains` for the number of chains,
                 `init_strategy` for the initialisation strategy for chains; `prior`
                 will draw init locations from prior, whereas `sir` will use Sequential-
-                Importance-Resampling using `init_strategy_num_candidates` to find init
-                locations.
-            get_potential_function: Callable that returns the potential function used
-                for MCMC sampling.
+                Importance-Resampling. Init strategies may have their own keywords
+                which can also be set from `mcmc_parameters`.
         """
         if method_family in ("snpe", "snle", "snre_a", "snre_b"):
             self._method_family = method_family
@@ -80,7 +79,6 @@ class NeuralPosterior(ABC):
         self.set_mcmc_method(mcmc_method)
         self.set_mcmc_parameters(mcmc_parameters)
 
-        self._get_potential_function = get_potential_function
         self._leakage_density_correction_factor = None  # Correction factor for SNPE.
         self._mcmc_init_params = None
         self._num_trained_rounds = 0
@@ -275,14 +273,14 @@ class NeuralPosterior(ABC):
     def _sample_posterior_mcmc(
         self,
         num_samples: int,
-        x: Tensor,
+        potential_fn: Callable,
+        init_fn: Optional[Callable] = None,
         mcmc_method: str = "slice_np",
         thin: int = 10,
         warmup_steps: int = 20,
         num_chains: Optional[int] = 1,
-        init_strategy: str = "prior",
-        init_strategy_num_candidates: int = 10000,
         show_progress_bars: bool = True,
+        **kwargs,
     ) -> Tensor:
         r"""
         Return MCMC samples from posterior $p(\theta|x)$.
@@ -293,7 +291,9 @@ class NeuralPosterior(ABC):
 
         Args:
             num_samples: Desired number of samples.
-            x: Conditioning context for posterior $p(\theta|x)$.
+            potential_fn: Potential function used for MCMC sampling.
+            init_fn: Initialisation function for each chain. When called without
+                arguments, it will return a single batched parameter set.
             mcmc_method: Sampling method. Currently defaults to `slice_np` for a custom
                 numpy implementation of slice sampling; select `hmc`, `nuts` or `slice`
                 for Pyro-based sampling.
@@ -301,65 +301,35 @@ class NeuralPosterior(ABC):
                 sample will be returned, until a total of `num_samples`.
             warmup_steps: Initial number of samples to discard.
             num_chains: Whether to sample in parallel. If None, use all but one CPU.
-            init_strategy: Initialisation strategy for chains; `prior` will draw init
-                locations from prior, whereas `sir` will use Sequential-Importance-
-                Resampling using `init_strategy_num_candidates` to find init
-                locations.
-            init_strategy_num_candidates: Number of candidate init locations
-                when `init_strategy` is `sir`.
             show_progress_bars: Whether to show a progressbar during sampling.
+            kwargs: Absorbs passed but unused arguments. E.g. in
+                `DirectPosterior.sample()` we pass `mcmc_parameters` which might
+                contain also entries for the mcmc initialization, which are not used
+                here.
 
         Returns:
             Tensor of shape (num_samples, shape_of_single_theta).
         """
-        # Find init points depending on `init_strategy` if no init is set
-        if self._mcmc_init_params is None:
-            if init_strategy == "prior":
-                self._mcmc_init_params = self._prior.sample((num_chains,)).detach()
-            elif init_strategy == "sir":
-                self.net.eval()
-                init_param_candidates = self._prior.sample(
-                    (init_strategy_num_candidates,)
-                ).detach()
-                potential_function = self._get_potential_function(
-                    self._prior, self.net, x, "slice_np"
-                )
-                log_weights = torch.cat(
-                    [
-                        potential_function(init_param_candidates[i, :]).detach()
-                        for i in range(init_strategy_num_candidates)
-                    ]
-                )
-                probs = np.exp(log_weights.view(-1).numpy().astype(np.float64))
-                probs[np.isnan(probs)] = 0.0
-                probs[np.isinf(probs)] = 0.0
-                probs /= probs.sum()
-                idxs = np.random.choice(
-                    a=np.arange(init_strategy_num_candidates),
-                    size=num_chains,
-                    replace=False,
-                    p=probs,
-                )
-                self._mcmc_init_params = init_param_candidates[
-                    torch.from_numpy(idxs.astype(int)), :
-                ]
-                self.net.train(True)
-            else:
-                raise NotImplementedError
 
-        potential_fn = self._get_potential_function(
-            self._prior, self.net, x, mcmc_method
-        )
-        track_gradients = mcmc_method != "slice" and mcmc_method != "slice_np"
+        initial_params = torch.cat([init_fn() for _ in range(num_chains)])
+
+        track_gradients = mcmc_method in ("hmc", "nuts")
         with torch.set_grad_enabled(track_gradients):
-            if mcmc_method == "slice_np":
+            if mcmc_method in ("slice_np", "slice_np_vectorized"):
                 samples = self._slice_np_mcmc(
-                    num_samples, potential_fn, thin, warmup_steps,
+                    num_samples=num_samples,
+                    potential_function=potential_fn,
+                    initial_params=initial_params,
+                    thin=thin,
+                    warmup_steps=warmup_steps,
+                    vectorized=(mcmc_method == "slice_np_vectorized"),
+                    show_progress_bars=show_progress_bars,
                 )
             elif mcmc_method in ("hmc", "nuts", "slice"):
                 samples = self._pyro_mcmc(
                     num_samples=num_samples,
                     potential_function=potential_fn,
+                    initial_params=initial_params,
                     mcmc_method=mcmc_method,
                     thin=thin,
                     warmup_steps=warmup_steps,
@@ -375,8 +345,11 @@ class NeuralPosterior(ABC):
         self,
         num_samples: int,
         potential_function: Callable,
+        initial_params: Tensor,
         thin: int,
         warmup_steps: int,
+        vectorized: bool = False,
+        show_progress_bars: bool = True,
     ) -> Tensor:
         """
         Custom implementation of slice sampling using Numpy.
@@ -384,53 +357,68 @@ class NeuralPosterior(ABC):
         Args:
             num_samples: Desired number of samples.
             potential_function: A callable **class**.
+            initial_params: Initial parameters for MCMC chain.
             thin: Thinning (subsampling) factor.
             warmup_steps: Initial number of samples to discard.
+            vectorized: Whether to use a vectorized implementation of
+                the Slice sampler (still experimental).
+            show_progress_bars: Whether to show a progressbar during sampling;
+                can only be turned off for vectorized sampler.
 
         Returns: Tensor of shape (num_samples, shape_of_single_theta).
         """
-        # Go into eval mode for evaluating during sampling
-        self.net.eval()
+        num_chains = initial_params.shape[0]
+        dim_samples = initial_params.shape[1]
 
-        num_chains = self._mcmc_init_params.shape[0]
-        dim_samples = self._mcmc_init_params.shape[1]
-
-        all_samples = []
-        for c in range(num_chains):
-            posterior_sampler = SliceSampler(
-                utils.tensor2numpy(self._mcmc_init_params[c, :]).reshape(-1),
-                lp_f=potential_function,
-                thin=thin,
+        if not vectorized:  # Sample all chains sequentially
+            all_samples = []
+            for c in range(num_chains):
+                posterior_sampler = SliceSampler(
+                    utils.tensor2numpy(initial_params[c, :]).reshape(-1),
+                    lp_f=potential_function,
+                    thin=thin,
+                )
+                if warmup_steps > 0:
+                    posterior_sampler.gen(int(warmup_steps))
+                all_samples.append(
+                    posterior_sampler.gen(ceil(num_samples / num_chains))
+                )
+            all_samples = np.stack(all_samples).astype(np.float32)
+            samples = torch.from_numpy(all_samples)  # chains x samples x dim
+        else:  # Sample all chains at the same time
+            posterior_sampler = SliceSamplerVectorized(
+                init_params=utils.tensor2numpy(initial_params),
+                log_prob_fn=potential_function,
+                num_chains=num_chains,
+                verbose=show_progress_bars,
             )
-            if warmup_steps > 0:
-                posterior_sampler.gen(int(warmup_steps))
-            all_samples.append(posterior_sampler.gen(int(num_samples / num_chains)))
-        all_samples = np.stack(all_samples).astype(np.float32)
+            warmup_ = warmup_steps * thin
+            num_samples_ = ceil((num_samples * thin) / num_chains)
+            samples = posterior_sampler.run(warmup_ + num_samples_)
+            samples = samples[:, warmup_:, :]  # discard warmup steps
+            samples = samples[:, ::thin, :]  # thin chains
+            samples = torch.from_numpy(samples)  # chains x samples x dim
 
-        samples = torch.from_numpy(all_samples)  # chains x samples x dim
-
-        # Final sample will be next init location
+        # Save sample as potential next init (if init_strategy == 'latest_sample').
         self._mcmc_init_params = samples[:, -1, :].reshape(num_chains, dim_samples)
 
         samples = samples.reshape(-1, dim_samples)[:num_samples, :]
         assert samples.shape[0] == num_samples
 
-        # Back to training mode
-        self.net.train(True)
-
         return samples.type(torch.float32)
 
     def _pyro_mcmc(
         self,
         num_samples: int,
         potential_function: Callable,
+        initial_params: Tensor,
         mcmc_method: str = "slice",
         thin: int = 10,
         warmup_steps: int = 200,
         num_chains: Optional[int] = 1,
         show_progress_bars: bool = True,
     ):
-        r"""Return samples obtained using Pyro HMC, NUTS or slice kernels.
+        r"""Return samples obtained using Pyro HMC, NUTS for slice kernels.
 
         Args:
             num_samples: Desired number of samples.
@@ -447,32 +435,168 @@ class NeuralPosterior(ABC):
         """
         num_chains = mp.cpu_count - 1 if num_chains is None else num_chains
 
-        # Go into eval mode for evaluating during sampling
-        self.net.eval()
-
         kernels = dict(slice=Slice, hmc=HMC, nuts=NUTS)
 
         sampler = MCMC(
             kernel=kernels[mcmc_method](potential_fn=potential_function),
             num_samples=(thin * num_samples) // num_chains + num_chains,
             warmup_steps=warmup_steps,
-            initial_params={"": self._mcmc_init_params},
+            initial_params={"": initial_params},
             num_chains=num_chains,
             mp_context="fork",
             disable_progbar=not show_progress_bars,
+            transforms={},
         )
         sampler.run()
         samples = next(iter(sampler.get_samples().values())).reshape(
-            -1, len(self._prior.mean)  # len(prior.mean) = dim of theta
+            -1, initial_params.shape[1]  # .shape[1] = dim of theta
         )
 
         samples = samples[::thin][:num_samples]
         assert samples.shape[0] == num_samples
 
-        # Back to training mode
+        return samples
+
+    def sample_conditional(
+        self,
+        potential_fn_provider: Callable,
+        sample_shape: Shape,
+        condition: Tensor,
+        dims_to_sample: List[int],
+        x: Optional[Tensor] = None,
+        show_progress_bars: bool = True,
+        mcmc_method: Optional[str] = None,
+        mcmc_parameters: Optional[Dict[str, Any]] = None,
+    ) -> Tensor:
+        r"""
+        Return samples from conditional posterior $p(\theta_i|\theta_j, x)$.
+
+        In this function, we do not sample from the full posterior, but instead only
+        from a few parameter dimensions while the other parameter dimensions are kept
+        fixed at values specified in `condition`.
+
+        Samples are obtained with MCMC.
+
+        Args:
+            potential_fn_provider: Returns the potential function for the unconditional
+                posterior.
+            sample_shape: Desired shape of samples that are drawn from posterior. If
+                sample_shape is multidimensional we simply draw `sample_shape.numel()`
+                samples and then reshape into the desired shape.
+            condition: Parameter set that all dimensions not specified in
+                `dims_to_sample` will be fixed to. Should contain dim_theta elements,
+                i.e. it could e.g. be a sample from the posterior distribution.
+                The entries at all `dims_to_sample` will be ignored.
+            dims_to_sample: Which dimensions to sample from. The dimensions not
+                specified in `dims_to_sample` will be fixed to values given in
+                `condition`.
+            x: Conditioning context for posterior $p(\theta|x)$. If not provided,
+                fall back onto `x_o` if previously provided for multiround training, or
+                to a set default (see `set_default_x()` method).
+            show_progress_bars: Whether to show sampling progress monitor.
+            mcmc_method: Optional parameter to override `self.mcmc_method`.
+            mcmc_parameters: Dictionary overriding the default parameters for MCMC.
+                The following parameters are supported: `thin` to set the thinning
+                factor for the chain, `warmup_steps` to set the initial number of
+                samples to discard, `num_chains` for the number of chains,
+                `init_strategy` for the initialisation strategy for chains; `prior`
+                will draw init locations from prior, whereas `sir` will use Sequential-
+                Importance-Resampling using `init_strategy_num_candidates` to find init
+                locations.
+
+        Returns:
+            Samples from conditional posterior.
+        """
+
+        x, num_samples, mcmc_method, mcmc_parameters = self._prepare_for_sample(
+            x, sample_shape, mcmc_method, mcmc_parameters
+        )
+
+        class PriorWithFewerDims:
+            """
+            Prior which samples only from the free dimensions of the conditional.
+
+            This is needed for the the MCMC initialization functions when conditioning.
+            For the prior init, we could post-hoc select the relevant dimensions. But
+            for SIR, we want to evaluate the `potential_fn` of the conditional
+            posterior, which takes only a subset of the full parameter vector theta
+            (only the `dims_to_sample`). This subset is provided by `.sample()` from
+            this class.
+            """
+
+            def __init__(self, full_prior: Any):
+                self.full_prior = full_prior
+
+            def sample(self, *args, **kwargs):
+                """
+                Sample only from the relevant dimension. Other dimensions are filled in
+                by the `ConditionalPotentialFunctionProvider()` during MCMC.
+                """
+                return self.full_prior.sample(*args, **kwargs)[:, dims_to_sample]
+
+            def log_prob(self, *args, **kwargs):
+                r"""
+                `log_prob` is same as for the full prior, because we usually evaluate
+                the $\theta$ under the full joint once we have added the condition.
+                """
+                return self.full_prior.log_prob(*args, **kwargs)
+
+        self.net.eval()
+
+        cond_potential_fn_provider = ConditionalPotentialFunctionProvider(
+            potential_fn_provider, condition, dims_to_sample
+        )
+
+        samples = self._sample_posterior_mcmc(
+            num_samples=num_samples,
+            potential_fn=cond_potential_fn_provider(
+                self._prior, self.net, x, mcmc_method
+            ),
+            init_fn=self._build_mcmc_init_fn(
+                PriorWithFewerDims(self._prior),
+                cond_potential_fn_provider(self._prior, self.net, x, "slice_np"),
+                **mcmc_parameters,
+            ),
+            mcmc_method=mcmc_method,
+            condition=condition,
+            dims_to_sample=dims_to_sample,
+            show_progress_bars=show_progress_bars,
+            **mcmc_parameters,
+        )
+
         self.net.train(True)
 
-        return samples
+        return samples.reshape((*sample_shape, -1))
+
+    def _build_mcmc_init_fn(
+        self,
+        prior: Any,
+        potential_fn: Callable,
+        init_strategy: str = "prior",
+        **kwargs,
+    ) -> Callable:
+        """
+        Return function that, when called, creates an initial parameter set for MCMC.
+
+        Args:
+            prior: Prior distribution.
+            potential_fn: Potential function that the candidate samples are weighted
+                with.
+            init_strategy: Specifies the initialization method. Either of
+                [`prior`|`sir`|`latest_sample`].
+            kwargs: Passed on to init function. This way, init specific keywords can
+                be set through `mcmc_parameters`. Unused arguments should be absorbed.
+
+        Returns: Initialization function.
+        """
+        if init_strategy == "prior":
+            return lambda: prior_init(prior, **kwargs)
+        elif init_strategy == "sir":
+            return lambda: sir(prior, potential_fn, **kwargs)
+        elif init_strategy == "latest_sample":
+            return lambda: self._mcmc_init_params
+        else:
+            raise NotImplementedError
 
     def _x_else_default_x(self, x: Optional[Array]) -> Array:
         if x is not None:
@@ -607,3 +731,84 @@ class NeuralPosterior(ABC):
         )
 
         return desc
+
+
+class ConditionalPotentialFunctionProvider:
+    """
+    Wraps the potential functions to allow for sampling from the conditional posterior.
+    """
+
+    def __init__(
+        self,
+        potential_fn_provider: Callable,
+        condition: Tensor,
+        dims_to_sample: List[int],
+    ):
+        """
+        Args:
+            potential_fn_provider: Creates potential function of unconditional
+                posterior.
+            condition: Parameter set that all dimensions not specified in
+                `dims_to_sample` will be fixed to. Should contain dim_theta elements,
+                i.e. it could e.g. be a sample from the posterior distribution.
+                The entries at all `dims_to_sample` will be ignored.
+            dims_to_sample: Which dimensions to sample from. The dimensions not
+                specified in `dims_to_sample` will be fixed to values given in
+                `condition`.
+        """
+
+        self.potential_fn_provider = potential_fn_provider
+        self.condition = ensure_theta_batched(condition)
+        self.dims_to_sample = dims_to_sample
+
+    def __call__(self, prior, net: nn.Module, x: Tensor, mcmc_method: str,) -> Callable:
+        """Return potential function.
+
+        Switch on numpy or pyro potential function based on `mcmc_method`.
+        """
+        # Set prior, net, and x as attributes of unconditional potential_fn_provider.
+        _ = self.potential_fn_provider.__call__(prior, net, x, mcmc_method)
+
+        if mcmc_method in ("slice", "hmc", "nuts"):
+            return self.pyro_potential
+        else:
+            return self.np_potential
+
+    def np_potential(self, theta: np.ndarray) -> ScalarFloat:
+        r"""
+        Return conditional posterior log-probability or $-\infty$ if outside prior.
+
+        Args:
+            theta: Free parameters $\theta_i$, batch dimension 1.
+
+        Returns:
+            Conditional posterior log-probability $\log(p(\theta_i|\theta_j, x))$,
+            masked outside of prior.
+        """
+        theta = torch.as_tensor(theta, dtype=torch.float32)
+
+        theta_condition = deepcopy(self.condition)
+        theta_condition[:, self.dims_to_sample] = theta
+
+        return self.potential_fn_provider.np_potential(
+            utils.tensor2numpy(theta_condition)
+        )
+
+    def pyro_potential(self, theta: Dict[str, Tensor]) -> Tensor:
+        r"""
+        Return conditional posterior log-probability or $-\infty$ if outside prior.
+
+        Args:
+            theta: Free parameters $\theta_i$ (from pyro sampler).
+
+        Returns:
+            Conditional posterior log-probability $\log(p(\theta_i|\theta_j, x))$,
+            masked outside of prior.
+        """
+
+        theta = next(iter(theta.values()))
+
+        theta_condition = deepcopy(self.condition)
+        theta_condition[:, self.dims_to_sample] = theta
+
+        return self.potential_fn_provider.pyro_potential({"": theta_condition})
