@@ -4,7 +4,7 @@
 
 from abc import ABC, abstractmethod
 from copy import deepcopy
-from typing import Any, Callable, Dict, Optional, Union, cast
+from typing import Any, Callable, Dict, List, Optional, Union, cast
 from warnings import warn
 
 import numpy as np
@@ -33,6 +33,7 @@ class PosteriorEstimator(NeuralInference, ABC):
         sample_with_mcmc: bool = False,
         mcmc_method: str = "slice_np",
         mcmc_parameters: Optional[Dict[str, Any]] = None,
+        rejection_sampling_parameters: Optional[Dict[str, Any]] = None,
         device: str = "cpu",
         logging_level: Union[int, str] = "WARNING",
         summary_writer: Optional[SummaryWriter] = None,
@@ -64,6 +65,11 @@ class PosteriorEstimator(NeuralInference, ABC):
                 draw init locations from prior, whereas `sir` will use
                 Sequential-Importance-Resampling using `init_strategy_num_candidates`
                 to find init locations.
+            rejection_sampling_parameters: Dictonary overriding the default parameters
+                for rejection sampling. The following parameters are supported:
+                `max_sampling_batch_size` to set the batch size for drawing new
+                samples from the candidate distribution, e.g., the posterior. Larger
+                batch size speeds up sampling.
 
         See docstring of `NeuralInference` class for all other arguments.
         """
@@ -94,6 +100,7 @@ class PosteriorEstimator(NeuralInference, ABC):
         self._sample_with_mcmc = sample_with_mcmc
         self._mcmc_method = mcmc_method
         self._mcmc_parameters = mcmc_parameters
+        self._rejection_sampling_parameters = rejection_sampling_parameters
 
         self._model_bank = []
         self.use_non_atomic_loss = False
@@ -188,7 +195,7 @@ class PosteriorEstimator(NeuralInference, ABC):
                 sample_with_mcmc=self._sample_with_mcmc,
                 mcmc_method=self._mcmc_method,
                 mcmc_parameters=self._mcmc_parameters,
-                get_potential_function=PotentialFunctionProvider(),
+                rejection_sampling_parameters=self._rejection_sampling_parameters,
             )
 
         # Fit posterior using newly aggregated data set.
@@ -387,80 +394,3 @@ class PosteriorEstimator(NeuralInference, ABC):
             log_prob = self._log_prob_proposal_posterior(theta, x, masks, proposal)
 
         return -(calibration_kernel(x) * log_prob)
-
-
-class PotentialFunctionProvider:
-    """
-    This class is initialized without arguments during the initialization of the
-    Posterior class. When called, it specializes to the potential function appropriate
-    to the requested `mcmc_method`.
-
-    NOTE: Why use a class?
-    ----------------------
-    During inference, we use deepcopy to save untrained posteriors in memory. deepcopy
-    uses pickle which can't serialize nested functions
-    (https://stackoverflow.com/a/12022055).
-
-    It is important to NOT initialize attributes upon instantiation, because we need the
-     most current trained posterior neural net.
-
-    Returns:
-        Potential function for use by either numpy or pyro sampler
-    """
-
-    def __call__(
-        self, prior, posterior_nn: nn.Module, x: Tensor, mcmc_method: str
-    ) -> Callable:
-        """Return potential function.
-
-        Switch on numpy or pyro potential function based on `mcmc_method`.
-
-        """
-        self.posterior_nn = posterior_nn
-        self.prior = prior
-        self.x = x
-
-        if mcmc_method in ("slice", "hmc", "nuts"):
-            return self.pyro_potential
-        else:
-            return self.np_potential
-
-    def np_potential(self, theta: np.ndarray) -> ScalarFloat:
-        r"""Return posterior theta log prob. $p(\theta|x)$, $-\infty$ if outside prior."
-
-        Args:
-            theta: Parameters $\theta$, batch dimension 1.
-
-        Returns:
-            Posterior log probability $\log(p(\theta|x))$.
-        """
-        theta = torch.as_tensor(theta, dtype=torch.float32)
-
-        is_within_prior = torch.isfinite(self.prior.log_prob(theta))
-        if is_within_prior:
-            target_log_prob = self.posterior_nn.log_prob(
-                inputs=theta.reshape(1, -1), context=self.x.reshape(1, -1),
-            )
-        else:
-            target_log_prob = -float("Inf")
-
-        return target_log_prob
-
-    def pyro_potential(self, theta: Dict[str, Tensor]) -> Tensor:
-        r"""Return posterior log prob. of theta $p(\theta|x)$, -inf where outside prior.
-
-        Args:
-            theta: Parameters $\theta$ (from pyro sampler).
-
-        Returns:
-            Posterior log probability $p(\theta|x)$, masked outside of prior.
-        """
-
-        theta = next(iter(theta.values()))
-        # Notice opposite sign to numpy.
-        log_prob_posterior = -self.posterior_nn.log_prob(inputs=theta, context=self.x)
-        log_prob_prior = self.prior.log_prob(theta)
-
-        within_prior = torch.isfinite(log_prob_prior)
-
-        return torch.where(within_prior, log_prob_posterior, log_prob_prior)
